/**************************************************************************************
Author:			David Szybala
Website:        www.davidszybala.com
Alt Website:   www.twistedgerbil.com
Program:        Color_Converter.ms
Date:            2/10/09
Description:    This script will convert color values
***************************************************************************************/

try (destroyDialog color_converter_rollout) catch()


struct TGSColorConverter
(
    /**************************************************************************
    Returns 'current_value' rounded to 'total_decimals'
    **************************************************************************/
    fn get_rounded_number current_value total_decimals =
    (
        local rounded_number = 0.0
        local decimal_points = 0
        
        decimal_points = pow 10 total_decimals
        rounded_number = current_value * decimal_points
        rounded_number = floor (rounded_number + 0.5)
        rounded_number = (rounded_number / decimal_points)
        
        rounded_number --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'hsv_value' as a point3 rgb value. HSV range 0 - 255, RGB range 0 - 255
    **************************************************************************/
    fn get_hsv_to_rgb hsv_value =
    (
        local temp_color = color 50 125 200
        local rgb_value = point3 0 0 0
        
        temp_color.h = hsv_value.x
        temp_color.s = hsv_value.y
        temp_color.v = hsv_value.z
        
        rgb_value.x = this.get_rounded_number temp_color.r 0
        rgb_value.y = this.get_rounded_number temp_color.g 0
        rgb_value.z = this.get_rounded_number temp_color.b 0
        
        rgb_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'rgb_value' as a point3 hsv value. RGB range 0 - 255, HSV range 0 - 255
    **************************************************************************/
    fn get_rgb_to_hsv rgb_value =
    (
        local temp_color = color rgb_value.x rgb_value.y rgb_value.z
        local hsv_value = point3 0 0 0
        
        hsv_value.x = this.get_rounded_number temp_color.h 0
        hsv_value.y = this.get_rounded_number temp_color.s 0
        hsv_value.z = this.get_rounded_number temp_color.v 0
        
        hsv_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'rgb_value' as a point3 hsv value. RGB range 0 - 255, HSV in degrees and percent
    **************************************************************************/
    fn get_rgb_to_hsv2 rgb_value =
    (
        local hsv_value = this.get_rgb_to_hsv rgb_value
        
        hsv_value.x = this.get_rounded_number ((hsv_value.x / 255.0) * 360) 0
        hsv_value.y = this.get_rounded_number ((hsv_value.y / 255.0) * 100) 0
        hsv_value.z = this.get_rounded_number ((hsv_value.z / 255.0) * 100) 0
        
        hsv_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'rgb_value' as a point3 xyz value. RGB range 0 - 1
    **************************************************************************/
    fn get_rgb_to_xyz rgb_value =
    (
        local xyz_value = point3 0 0 0
        
        if rgb_value.x > 0.04045 then
            rgb_value.x = pow ((rgb_value.x + 0.055) / 1.055) 2.4
        else
            rgb_value.x /= 12.92
        
        if rgb_value.y > 0.04045 then
            rgb_value.y = pow ((rgb_value.y + 0.055) / 1.055) 2.4
        else
            rgb_value.y /= 12.92
        
        if rgb_value.z > 0.04045 then
            rgb_value.z = pow ((rgb_value.z + 0.055) / 1.055) 2.4
        else
            rgb_value.z /= 12.92
        
        rgb_value *= 100
        
        --Observer. = 2, Illuminant = D65
        xyz_value.x = rgb_value.x * 0.4124 + rgb_value.y * 0.3576 + rgb_value.z * 0.1805
        xyz_value.y = rgb_value.x * 0.2126 + rgb_value.y * 0.7152 + rgb_value.z * 0.0722
        xyz_value.z = rgb_value.x * 0.0193 + rgb_value.y * 0.1192 + rgb_value.z * 0.9505
        
        xyz_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'xyz_value' as a point3 rgb value. RGB range 0 - 255
    **************************************************************************/
    fn get_xyz_to_rgb xyz_value =
    (
        local rgb_value = point3 0 0 0
        
        --Observer = 2, Illuminant = D65
        xyz_value.x /= 100.0
        xyz_value.y /= 100.0
        xyz_value.z /= 100.0
        
        rgb_value.x = xyz_value.x *  3.2406 + xyz_value.y * -1.5372 + xyz_value.z * -0.4986
        rgb_value.y = xyz_value.x * -0.9689 + xyz_value.y *  1.8758 + xyz_value.z *  0.0415
        rgb_value.z = xyz_value.x *  0.0557 + xyz_value.y * -0.2040 + xyz_value.z *  1.0570
        
        if rgb_value.x > 0.0031308 then
            rgb_value.x = 1.055 * (pow rgb_value.x (1/2.4)) - 0.055
        else
            rgb_value.x *= 12.92
        
        if rgb_value.y > 0.0031308 then
            rgb_value.y = 1.055 * (pow rgb_value.y (1/2.4)) - 0.055
        else
            rgb_value.y *= 12.92
        
        if rgb_value.z > 0.0031308 then
            rgb_value.z = 1.055 * (pow rgb_value.z (1/2.4)) - 0.055
        else
            rgb_value.z *= 12.92
        
        rgb_value *= 255
        
        rgb_value.x = this.get_rounded_number rgb_value.x 0
        rgb_value.y = this.get_rounded_number rgb_value.y 0
        rgb_value.z = this.get_rounded_number rgb_value.z 0
        
        rgb_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'lab_value' as a point3 xyz value
    **************************************************************************/
    fn get_lab_to_xyz lab_value =
    (
        local xyz_value = point3 0 0 0
        
        xyz_value.y = (lab_value.x + 16) / 116.0
        xyz_value.x = lab_value.y / 500.0 + xyz_value.y
        xyz_value.z = xyz_value.y - lab_value.z / 200.0
        
        if ((pow xyz_value.x 3) > 0.008856) then
            xyz_value.x = pow xyz_value.x 3
        else
            xyz_value.x = (xyz_value.x - 16 / 116.0) / 7.787
        
        if ((pow xyz_value.y 3) > 0.008856) then
            xyz_value.y = pow xyz_value.y 3
        else
            xyz_value.y = (xyz_value.y - 16 / 116.0) / 7.787
        
        if ((pow xyz_value.z 3) > 0.008856) then
            xyz_value.z = pow xyz_value.z 3
        else
            xyz_value.z = (xyz_value.z - 16 / 116.0) / 7.787
        
        --Observer= 2, Illuminant= D65
        xyz_value.x *= 95.047
        xyz_value.y *= 100.000
        xyz_value.z *= 108.883
        
        xyz_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'xyz_value' as a point3 lab value
    **************************************************************************/
    fn get_xyz_to_lab xyz_value =
    (
        local lab_value = point3 0 0 0
        
        --Observer= 2, Illuminant= D65
        xyz_value.x /= 95.047
        xyz_value.y /= 100.000
        xyz_value.z /= 108.883
        
        if xyz_value.x > 0.008856 then
            xyz_value.x = pow xyz_value.x (1/3.0)
        else
            xyz_value.x = (7.787 * xyz_value.x) + (16 / 116.0)
        
        if xyz_value.y > 0.008856 then
            xyz_value.y = pow xyz_value.y (1/3.0)
        else
            xyz_value.y = (7.787 * xyz_value.y) + (16 / 116.0)
        
        if xyz_value.z > 0.008856 then
            xyz_value.z = pow xyz_value.z (1/3.0)
        else
            xyz_value.z = (7.787 * xyz_value.z) + (16 / 116.0)
        
        lab_value.x = (116 * xyz_value.y) - 16
        lab_value.y = 500 * (xyz_value.x - xyz_value.y)
        lab_value.z = 200 * (xyz_value.y - xyz_value.z)
        
        lab_value.x = this.get_rounded_number lab_value.x 0
        lab_value.y = this.get_rounded_number lab_value.y 0
        lab_value.z = this.get_rounded_number lab_value.z 0
        
        lab_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'rgb_value' as a point3 cmy value. RGB range 0 - 1, CMY range 0 - 1
    **************************************************************************/
    fn get_rgb_to_cmy rgb_value =
    (
        local cmy_value = point3 0 0 0
        
        cmy_value.x = 1 - rgb_value.x
        cmy_value.y = 1 - rgb_value.y
        cmy_value.z = 1 - rgb_value.z
        
        cmy_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'cmy_value' as a point3 rgb value. CMY range 0 - 1, RGB range 0 - 255
    **************************************************************************/
    fn get_cmy_to_rgb cmy_value =
    (
        local rgb_value = point3 0 0 0
        
        rgb_value.x = (1 - cmy_value.x) * 255
        rgb_value.y = (1 - cmy_value.y) * 255
        rgb_value.z = (1 - cmy_value.z) * 255
        
        rgb_value.x = this.get_rounded_number rgb_value.x 0
        rgb_value.y = this.get_rounded_number rgb_value.y 0
        rgb_value.z = this.get_rounded_number rgb_value.z 0
        
        rgb_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'cmy_value' as a point4 cmyk value. CMY range 0 - 1, CMYK is percentage
    **************************************************************************/
    fn get_cmy_to_cmyk cmy_value =
    (
        local cmyk_value = point4 0 0 0 0
        
        cmyk_value.w = amin #(cmy_value.x, cmy_value.y, cmy_value.z, 1.0)
        cmyk_value.x = (cmy_value.x - cmyk_value.w) / (1 - cmyk_value.w)
        cmyk_value.y = (cmy_value.y - cmyk_value.w) / (1 - cmyk_value.w)
        cmyk_value.z = (cmy_value.z - cmyk_value.w) / (1 - cmyk_value.w)
        
        --Convert to percentage
        cmyk_value *= 100
        cmyk_value.x = this.get_rounded_number cmyk_value.x 0
        cmyk_value.y = this.get_rounded_number cmyk_value.y 0
        cmyk_value.z = this.get_rounded_number cmyk_value.z 0
        cmyk_value.w = this.get_rounded_number cmyk_value.w 0
        
        cmyk_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point4 'cmyk_value' as a point3 cmy value. CMYK is percentage, CMY range 0 - 1
    **************************************************************************/
    fn get_cmyk_to_cmy cmyk_value =
    (
        local cmy_value = point3 0 0 0
        
        --CRITICAL! Convert to 0 - 1 range before calculation!
        cmyk_value /= 100.0
        
        cmy_value.x = (cmyk_value.x * (1 - cmyk_value.w) + cmyk_value.w)
        cmy_value.y = (cmyk_value.y * (1 - cmyk_value.w) + cmyk_value.w)
        cmy_value.z = (cmyk_value.z * (1 - cmyk_value.w) + cmyk_value.w)
        
        cmy_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the point3 'rgb_value' as a string hex value. RGB range 0 - 255
    **************************************************************************/
    fn get_rgb_to_hex rgb_value =
    (
        local hex_value = ""
        local conversions = #(bit.intAsHex  rgb_value.x, bit.intAsHex  rgb_value.y, bit.intAsHex  rgb_value.z)
        
        for current in conversions do
        (
            if current.count == 1 then
                hex_value += ("0" + current)
            else
                hex_value += current
        ) --end for
        
        hex_value = toUpper hex_value
        
        hex_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the string 'hex_value' as a point3 rgb value. RGB range 0 - 255
    **************************************************************************/
    fn get_hex_to_rgb hex_value =
    (
        local rgb_value = point3 0 0 0
        
        rgb_value.x = ("0x" + (substring hex_value 1 2)) as integer
        rgb_value.y = ("0x" + (substring hex_value 3 2)) as integer
        rgb_value.z = ("0x" + (substring hex_value 5 2)) as integer
        
        rgb_value --return value
    ), --end fn
    
    /**************************************************************************
    Returns the color 'current_color' wth a gamma value applied. RGB range 0 - 255
    **************************************************************************/
    fn get_gamma_color current_color gamma_value degamma =
    (
        local gamma_color = copy current_color
        local gamma_float = point3 (gamma_color.r / 255.0) (gamma_color.g / 255.0) (gamma_color.b / 255.0)
        
        if degamma then
        (
            gamma_float.x = pow gamma_float.x gamma_value
            gamma_float.y = pow gamma_float.y gamma_value
            gamma_float.z = pow gamma_float.z gamma_value
        ) --end if
        else
        (
            gamma_float.x = pow gamma_float.x (1 / gamma_value)
            gamma_float.y = pow gamma_float.y (1 / gamma_value)
            gamma_float.z = pow gamma_float.z (1 / gamma_value)
        ) --end else
        
        gamma_float *= 255
        
        gamma_float.x = this.get_rounded_number gamma_float.x 0
        gamma_float.y = this.get_rounded_number gamma_float.y 0
        gamma_float.z = this.get_rounded_number gamma_float.z 0
        
        gamma_color = color (gamma_float.x as integer) (gamma_float.y as integer) (gamma_float.z as integer)
        
        gamma_color --return value
    ) --end fn
) --end struct


(--put script into local scope
    global color_converter_rollout
    local color_converter_ini_file = getDir #plugcfg + "\\TGSColorConverter.ini"
    local color_converter_rollout_pos
    
    rollout color_converter_rollout "Color Converter"
    (
        local converter = TGSColorConverter()
        
        /**********************************************************************
        Sets the RGB group controls
        **********************************************************************/
        fn set_rgb r_value g_value b_value =
        (
            if r_value > 255 do r_value = 255
            if r_value < 0 do r_value = 0
            color_converter_rollout.red_integer_spinner.value = r_value as integer
            color_converter_rollout.red_float_spinner.value = r_value / 255.0
            
            if g_value > 255 do g_value = 255
            if g_value < 0 do g_value = 0
            color_converter_rollout.green_integer_spinner.value = g_value as integer
            color_converter_rollout.green_float_spinner.value = g_value / 255.0
            
            if b_value > 255 do b_value = 255
            if b_value < 0 do b_value = 0
            color_converter_rollout.blue_integer_spinner.value = b_value as integer
            color_converter_rollout.blue_float_spinner.value = b_value / 255.0
        )--end fn
        
        /**********************************************************************
        Sets the RGB controls from the HSV controls
        **********************************************************************/
        fn set_rgb_from_hsv =
        (
            local hsv_value = point3 0 0 0
            local rgb_value = point3 0 0 0
            
            hsv_value.x = color_converter_rollout.hue_integer_spinner.value
            hsv_value.y = color_converter_rollout.saturation_integer_spinner.value
            hsv_value.z = color_converter_rollout.value_integer_spinner.value
            
            rgb_value = converter.get_hsv_to_rgb hsv_value
            
            set_rgb rgb_value.x rgb_value.y rgb_value.z
        ) --end fn
        
        /**********************************************************************
        Sets the RGB controls from the LAB controls
        **********************************************************************/
        fn set_rgb_from_lab =
        (
            local lab_value = point3 0 0 0
            local xyz_value = point3 0 0 0
            local rgb_value = point3 0 0 0
            
            lab_value.x = color_converter_rollout.lab_l_spinner.value
            lab_value.y = color_converter_rollout.lab_a_spinner.value
            lab_value.z = color_converter_rollout.lab_b_spinner.value
            
            xyz_value = converter.get_lab_to_xyz lab_value
            rgb_value = converter.get_xyz_to_rgb xyz_value
            
            set_rgb rgb_value.x rgb_value.y rgb_value.z
        ) --end fn
        
        /**********************************************************************
        Sets the RGB controls from the CMYK controls
        **********************************************************************/
        fn set_rgb_from_cmyk =
        (
            local cmyk_value = point4 0 0 0 0
            local cmy_value = point3 0 0 0
            local rgb_value = point3 0 0 0
            
            cmyk_value.x = color_converter_rollout.cmyk_c_spinner.value
            cmyk_value.y = color_converter_rollout.cmyk_m_spinner.value
            cmyk_value.z = color_converter_rollout.cmyk_y_spinner.value
            cmyk_value.w = color_converter_rollout.cmyk_k_spinner.value
             
            cmy_value = converter.get_cmyk_to_cmy cmyk_value
            rgb_value = converter.get_cmy_to_rgb cmy_value
            
            set_rgb rgb_value.x rgb_value.y rgb_value.z
        ) --end fn
        
        /**********************************************************************
        Sets the RGB controls from the HEX value
        **********************************************************************/
        fn set_rgb_from_hex =
        (
            local rgb_value = converter.get_hex_to_rgb color_converter_rollout.hex_text.text
            
            set_rgb rgb_value.x rgb_value.y rgb_value.z
        ) --end fn
        
        /**********************************************************************
        Sets the HSV controls from the RGB controls
        **********************************************************************/
        fn set_hsv_from_rgb =
        (
            local rgb_value = point3 0 0 0
            local hsv_value = point3 0 0 0
            
            rgb_value.x = color_converter_rollout.red_integer_spinner.value
            rgb_value.y = color_converter_rollout.green_integer_spinner.value
            rgb_value.z = color_converter_rollout.blue_integer_spinner.value
            
            hsv_value = converter.get_rgb_to_hsv rgb_value
            
            color_converter_rollout.hue_integer_spinner.value = hsv_value.x as integer
            color_converter_rollout.saturation_integer_spinner.value = hsv_value.y as integer
            color_converter_rollout.value_integer_spinner.value = hsv_value.z as integer
            
            hsv_value = converter.get_rgb_to_hsv2 rgb_value
            
            color_converter_rollout.hue_degree_spinner.value = hsv_value.x as integer
            color_converter_rollout.saturation_percent_spinner.value = hsv_value.y as integer
            color_converter_rollout.value_percent_spinner.value = hsv_value.z as integer
        ) --end fn
        
        /**********************************************************************
        Sets the RGB controls from the LAB controls
        **********************************************************************/
        fn set_lab_from_rgb =
        (
            local rgb_value = point3 0 0 0
            local xyz_value = point3 0 0 0
            local lab_value = point3 0 0 0
            
            rgb_value.x = color_converter_rollout.red_float_spinner.value
            rgb_value.y = color_converter_rollout.green_float_spinner.value
            rgb_value.z = color_converter_rollout.blue_float_spinner.value
            
            xyz_value = converter.get_rgb_to_xyz rgb_value
            lab_value = converter.get_xyz_to_lab xyz_value

            color_converter_rollout.lab_l_spinner.value = lab_value.x as integer
            color_converter_rollout.lab_a_spinner.value = lab_value.y as integer
            color_converter_rollout.lab_b_spinner.value = lab_value.z as integer
        ) --end fn
        
        /**********************************************************************
        Sets the CMYK controls from the RGB controls
        **********************************************************************/
        fn set_cymk_from_rgb =
        (
            local rgb_value = point3 0 0 0
            local cmy_value = point3 0 0 0
            local cmyk_value = point4 0 0 0 0
            
            rgb_value.x = color_converter_rollout.red_float_spinner.value
            rgb_value.y = color_converter_rollout.green_float_spinner.value
            rgb_value.z = color_converter_rollout.blue_float_spinner.value
            
            cmy_value = converter.get_rgb_to_cmy rgb_value
            cmyk_value = converter.get_cmy_to_cmyk cmy_value
            
            color_converter_rollout.cmyk_c_spinner.value = cmyk_value.x as integer
            color_converter_rollout.cmyk_m_spinner.value = cmyk_value.y as integer
            color_converter_rollout.cmyk_y_spinner.value = cmyk_value.z as integer
            color_converter_rollout.cmyk_k_spinner.value = cmyk_value.w as integer
        ) --end fn
        
        /**********************************************************************
        Sets the HEX controls from the RGB controls
        **********************************************************************/
        fn set_hex_from_rgb =
        (
            local rgb_value = point3 0 0 0
            local hex_value = ""
            
            rgb_value.x = color_converter_rollout.red_integer_spinner.value
            rgb_value.y = color_converter_rollout.green_integer_spinner.value
            rgb_value.z = color_converter_rollout.blue_integer_spinner.value
            
            hex_value = converter.get_rgb_to_hex rgb_value
            
            color_converter_rollout.hex_text.text = hex_value
        ) --end fn
        
        /**********************************************************************
        Sets the gamma color from the result color and gamma controls
        **********************************************************************/
        fn set_gamma_result =
        (
            local result_color = color_converter_rollout.result_color.color
            local gamma_value = color_converter_rollout.gamma_spinner.value
            local degamma = color_converter_rollout.gamma_invert_checkbox.checked
            
            color_converter_rollout.gamma_color.color = converter.get_gamma_color result_color gamma_value degamma
        ) --end fn
        
        /**********************************************************************
        Sets the results of all color conversions
        **********************************************************************/
        fn set_conversion_results set_hsv set_lab set_cmyk set_hex =
        (
            if set_hsv do
                set_hsv_from_rgb()
            
            if set_lab do
                set_lab_from_rgb()
                
            if set_cmyk do
                set_cymk_from_rgb()
            
            if set_hex do
                set_hex_from_rgb()
            
            local result_color = color 0 0 0
            result_color.r = color_converter_rollout.red_integer_spinner.value
            result_color.g = color_converter_rollout.green_integer_spinner.value
            result_color.b = color_converter_rollout.blue_integer_spinner.value
            
            color_converter_rollout.result_color.color = result_color
            
            set_gamma_result()
        )--end fn
    
        /**********************************************************************
        UI Creation
        **********************************************************************/
        groupbox rgb_group "RGB" pos:[5,5] width:170 height:80
            label red_label "R:" pos:[15,26] fieldwidth:10
            label green_label "G:" pos:[15,46] width:10
            label blue_label "B:" pos:[15,66] width:10
            spinner red_integer_spinner "" pos:[30,25] width:45 range:[0,255,127] type:#integer
            spinner green_integer_spinner "" pos:[30,45] width:45 range:[0,255,127] type:#integer
            spinner blue_integer_spinner "" pos:[30,65] width:45 range:[0,255,127] type:#integer
            label red_equal_label "=" pos:[85,26] width:5
            label green_equal_label "=" pos:[85,46] width:5
            label blue_equal_label "=" pos:[85,66] width:5
            spinner red_float_spinner "" pos:[100,25] width:60 range:[0,1,0.498] type:#float scale:0.01
            spinner green_float_spinner "" pos:[100,45] width:60 range:[0,1,0.498] type:#float scale:0.01
            spinner blue_float_spinner "" pos:[100,65] width:60 range:[0,1,0.498] type:#float scale:0.01
            
        groupbox hsv_group "HSV" pos:[5,90] width:170 height:80
            label hue_label "H:" pos:[15,111] width:10
            label saturation_label "S:" pos:[15,131] width:10
            label value_label "V:" pos:[15,151] width:10
            spinner hue_integer_spinner "" pos:[30,110] width:45 range:[0,255,0] type:#integer
            spinner saturation_integer_spinner "" pos:[30,130] width:45 range:[0,255,0] type:#integer
            spinner value_integer_spinner "" pos:[30,150] width:45 range:[0,255,127] type:#integer
            label hue_equal_label "=" pos:[85,111] width:5
            label sat_equal_label "=" pos:[85,131] width:5
            label value_equal_label "=" pos:[85,151] width:5
            spinner hue_degree_spinner "" pos:[100,110] width:45 range:[0,360,0] type:#integer
            spinner saturation_percent_spinner "" pos:[100,130] width:45 range:[0,100,0] type:#integer
            spinner value_percent_spinner "" pos:[100,150] width:45 range:[0,100,50] type:#integer
            label hue_degree_label "" pos:[150,111] width:10
            label sat_percent_label "%" pos:[150,131] width:10
            label value_percent_label "%" pos:[150,151] width:10
            
        groupbox lab_group "Lab" pos:[5,175] width:80 height:80
            label lab_l_label "L:" pos:[15,196] width:10
            label lab_a_label "a:" pos:[15,216] width:10
            label lab_b_label "b:" pos:[15,236] width:10
            spinner lab_l_spinner "" pos:[30,195] width:45 range:[0,100,53] type:#integer
            spinner lab_a_spinner "" pos:[30,215] width:45 range:[-127,127,0] type:#integer
            spinner lab_b_spinner "" pos:[30,235] width:45 range:[-127,127,0] type:#integer
            
        groupbox cmyk_group "CMYK (%)" pos:[95,175] width:80 height:100
            label cmyk_c_label "C:" pos:[105,196] width:10
            label cmyk_m_label "M:" pos:[105,216] width:10
            label cmyk_y_label "Y:" pos:[105,236] width:10
            label cmyk_k_label "K:" pos:[105,256] width:10
            spinner cmyk_c_spinner "" pos:[120,196] width:45 range:[0,100,0] type:#integer
            spinner cmyk_m_spinner "" pos:[120,216] width:45 range:[0,100,0] type:#integer
            spinner cmyk_y_spinner "" pos:[120,236] width:45 range:[0,100,0] type:#integer
            spinner cmyk_k_spinner "" pos:[120,256] width:45 range:[0,100,50] type:#integer
        
        groupbox hex_group "Hex" pos:[5,260] width:80 height:40
            edittext hex_text "#" pos:[10,278] width:65 height:17 text:"7F7F7F"
            
        groupbox gamma_group "Gamma Correction" pos:[5,305] width:170 height:40
            spinner gamma_spinner "" pos:[15,325] width:45 range:[1.0,10,2.2] type:#float scale:0.1
            checkbox gamma_invert_checkbox "De-gamma" pos:[70,326] width:75 height:15 checked:true
        
        groupbox result_group "Results" pos:[5,350] width:170 height:45
            colorPicker result_color "" pos:[15,370] width:70 height:20 color:(color 127 127 127)
            colorPicker gamma_color "" pos:[90,370] width:70 height:20 color:(color 55 55 55)
        
        /**********************************************************************
        UI open event handler. Refreshes the material list
        **********************************************************************/
		on color_converter_rollout open do
            hue_degree_label.text = (bit.intasChar 186)
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on color_converter_rollout moved pos do
            setIniSetting color_converter_ini_file "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        RGB spinner event handlers. Refreshes all color values
        **********************************************************************/
        on red_integer_spinner changed theValue do
        (
            red_float_spinner.value = (theValue / 255.0)
            set_conversion_results true true true true --hsv, lab, cmyk, hex
        )--end on
        
        on green_integer_spinner changed theValue do
        (
            green_float_spinner.value = (theValue / 255.0)
            set_conversion_results true true true true --hsv, lab, cmyk, hex
        )--end on
        
        on blue_integer_spinner changed theValue do
        (
            blue_float_spinner.value = (theValue / 255.0)
            set_conversion_results true true true true --hsv, lab, cmyk, hex
        )--end on
        
        on red_float_spinner changed theValue do
        (
            red_integer_spinner.value = (theValue * 255.0) as integer
            set_conversion_results true true true true --hsv, lab, cmyk, hex
        )--end on
        
        on green_float_spinner changed theValue do
        (
            green_integer_spinner.value = (theValue * 255.0) as integer
            set_conversion_results true true true true --hsv, lab, cmyk, hex
        )--end on
        
        on blue_float_spinner changed theValue do
        (
            blue_integer_spinner.value = (theValue * 255.0) as integer
            set_conversion_results true true true true --hsv, lab, cmyk, hex
        )--end on
        
        /**********************************************************************
        HSV spinner event handlers. Sets the RGB values, then refreshes all other values
        **********************************************************************/
        on hue_integer_spinner changed new_value do
        (
            new_value = (new_value / 255.0) * 360
            new_value = converter.get_rounded_number new_value 0
            hue_degree_spinner.value = new_value as integer
            
            set_rgb_from_hsv()
            set_conversion_results false true true true --hsv, lab, cmyk, hex
        )--end on
        
        on saturation_integer_spinner changed new_value do
        (
            new_value = (new_value / 255.0) * 100
            new_value = converter.get_rounded_number new_value 0
            saturation_percent_spinner.value = new_value as integer
            
            set_rgb_from_hsv()
            set_conversion_results false true true true --hsv, lab, cmyk, hex
        )--end on
        
        on value_integer_spinner changed new_value do
        (
            new_value = (new_value / 255.0) * 100
            new_value = converter.get_rounded_number new_value 0
            value_percent_spinner.value = new_value as integer
            
            set_rgb_from_hsv()
            set_conversion_results false true true true --hsv, lab, cmyk, hex
        )--end on
        
        on hue_degree_spinner changed new_value do
        (
            new_value = (new_value / 360.0) * 255
            new_value = converter.get_rounded_number new_value 0
            hue_integer_spinner.value = new_value as integer
            
            set_rgb_from_hsv()
            set_conversion_results false true true true --hsv, lab, cmyk, hex
        )--end on
        
        on saturation_percent_spinner changed new_value do
        (
            new_value = (new_value / 100.0) * 255
            new_value = converter.get_rounded_number new_value 0
            saturation_integer_spinner.value = new_value as integer
            
            set_rgb_from_hsv()
            set_conversion_results false true true true --hsv, lab, cmyk, hex
        )--end on
        
        on value_percent_spinner changed new_value do
        (
            new_value = (new_value / 100.0) * 255
            new_value = converter.get_rounded_number new_value 0
            value_integer_spinner.value = new_value as integer
            
            set_rgb_from_hsv()
            set_conversion_results false true true true --hsv, lab, cmyk, hex
        )--end on
        
        /**********************************************************************
        LAB spinner event handlers. Sets the RGB values, then refreshes all other values
        **********************************************************************/
        on lab_l_spinner changed new_value do
        (
            set_rgb_from_lab()
            set_conversion_results true false true true --hsv, lab, cmyk, hex
        )--end on
        
        on lab_a_spinner changed new_value do
        (
            set_rgb_from_lab()
            set_conversion_results true false true true --hsv, lab, cmyk, hex
        )--end on 
        
        on lab_b_spinner changed new_value do
        (
            set_rgb_from_lab()
            set_conversion_results true false true true --hsv, lab, cmyk, hex
        )--end on
        
        /**********************************************************************
        CMYK spinner event handlers. Sets the RGB values, then refreshes all other values
        **********************************************************************/
        on cmyk_c_spinner changed new_value do
        (
            set_rgb_from_cmyk()
            set_conversion_results true true false true --hsv, lab, cmyk, hex
        ) --end on
        
        on cmyk_m_spinner changed new_value do
        (
            set_rgb_from_cmyk()
            set_conversion_results true true false true --hsv, lab, cmyk, hex
        ) --end on
        
        on cmyk_y_spinner changed new_value do
        (
            set_rgb_from_cmyk()
            set_conversion_results true true false true --hsv, lab, cmyk, hex
        ) --end on
        
        on cmyk_k_spinner changed new_value do
        (
            set_rgb_from_cmyk()
            set_conversion_results true true false true --hsv, lab, cmyk, hex
        ) --end on
        
        /**********************************************************************
        Hex spinner event handlers. Sets the RGB values, then refreshes all other values
        **********************************************************************/
        on hex_text entered new_text do
        (
            local valid_characters = #("A","B","C","D","E","F","1","2","3","4","5","6","7","8","9","0")
            local is_valid = true
            
            new_text = toUpper new_text
            hex_text.text = new_text
            
            if new_text.count != 6 do
                is_valid = false
            
            for i = 1 to 6 while is_valid do
            (
                if (findItem valid_characters new_text[i]) == 0 do
                    is_valid = false
            ) --end for
            
            if is_valid then
            (
                set_rgb_from_hex()
                set_conversion_results true true true false --hsv, lab, cmyk, hex
            ) --end if
            else
                hex_text.text = "INVALID"
        )--end on
        
        /**********************************************************************
        Gamma spinner event handlers. Adjusts the gamma of the result color
        **********************************************************************/
        on gamma_spinner changed the_value do
            set_gamma_result()
        
        on gamma_invert_checkbox changed the_state do
            set_gamma_result()
    )--end rollout
    
    color_converter_rollout_pos = execute (getIniSetting color_converter_ini_file "Dialog" "Position")
    if color_converter_rollout_pos == OK do color_converter_rollout_pos = [100,100]
    if color_converter_rollout_pos.x > sysinfo.desktopsize.x - 155 do
        color_converter_rollout_pos.x = sysinfo.desktopsize.x - 160
    if color_converter_rollout_pos.y > sysinfo.desktopsize.y - 415 do
        color_converter_rollout_pos.y = sysinfo.desktopsize.y - 415
    if color_converter_rollout_pos.x < 0 do color_converter_rollout_pos.x = 0 
    if color_converter_rollout_pos.y < 0 do color_converter_rollout_pos.y = 0
    createDialog color_converter_rollout 180 400 color_converter_rollout_pos.x color_converter_rollout_pos.y
)--end local scope
